#!/bin/sh


if [ x"$ASPEN_GPU_ID" = x ]; then
	export ASPEN_GPU_ID=0
fi
if [ x"$CUDA_PATH" = x ] || [ ! -e $CUDA_PATH ]; then
	export CUDA_PATH=/usr/local/cuda
fi

export LD_LIBRARY_PATH=$CUDA_PATH/lib64:$LD_LIBRARY_PATH
export LD_LIBRARY_PATH=`pwd`/../lib:$LD_LIBRARY_PATH

if [ x"$OMP_NUM_THREADS" != x ]; then
	unset OMP_NUM_THREADS
fi

IN=IN-HPC151
IN_FROM=16
IN_TO=20000
IN_STEP=16
awk 'BEGIN{ print 1; for(i='$IN_FROM';i<='$IN_TO';i+='$IN_STEP'){ print i; } print -1; exit; }' > $IN

GPU=`../src/get_dev_info | awk '/DEVICE/{ gsub(/^[A-Za-z0-9]*-/,"",$2); gsub(/-/,""); print $2; }'`
DRIVER=`head -1 /proc/driver/nvidia/version | awk '{ gsub(/.*Module/,""); print $1}'`

EXEC='timeout -s KILL 36000 '
OPTIONAL='-'$DRIVER

ASPEN_V=1.6p1
CUDA_V=9.0RC
MAGMA_V=2.1.0
KBLAS_V=1.3-beta

ASPEN_LIB_PATH=/home/ASPEN.K2-$ASPEN_V/lib
MAGMA_LIB_PATH=/opt/tmp/magma-$MAGMA_V/lib
KBLAS_LIB_PATH=/opt/tmp/kblas-$KBLAS_V/lib

export LD_LIBRARY_PATH=$ASPEN_LIB_PATH:$MAGMA_LIB_PATH:$KBLAS_LIB_PATH:$LD_LIBRARY_PATH


EXEC_ASPEN=1
EXEC_CUDA=1
# MAGMA and KBLAS are optional
# You need to prepare lib archives in advance.
EXEC_MAGMA=0
EXEC_KBLAS=0



if [ $EXEC_ASPEN -ne 0 ]; then
#$EXEC ./test2-ssymv-u $IN | tee log-ASPEN.K2-$ASPEN_V-SSYMVU-$GPU$OPTIONAL
#$EXEC ./test2-ssymv-l $IN | tee log-ASPEN.K2-$ASPEN_V-SSYMVL-$GPU$OPTIONAL
#$EXEC ./test2-dsymv-u $IN | tee log-ASPEN.K2-$ASPEN_V-DSYMVU-$GPU$OPTIONAL
#$EXEC ./test2-dsymv-l $IN | tee log-ASPEN.K2-$ASPEN_V-DSYMVL-$GPU$OPTIONAL
#$EXEC ./test2-wsymv-u $IN | tee log-ASPEN.K2-$ASPEN_V-WSYMVU-$GPU$OPTIONAL
$EXEC ./test2-wsymv-l $IN | tee log-ASPEN.K2-$ASPEN_V-WSYMVL-$GPU$OPTIONAL
$EXEC ./test2-chemv-u $IN | tee log-ASPEN.K2-$ASPEN_V-CHEMVU-$GPU$OPTIONAL
$EXEC ./test2-chemv-l $IN | tee log-ASPEN.K2-$ASPEN_V-CHEMVL-$GPU$OPTIONAL
$EXEC ./test2-zhemv-u $IN | tee log-ASPEN.K2-$ASPEN_V-ZHEMVU-$GPU$OPTIONAL
$EXEC ./test2-zhemv-l $IN | tee log-ASPEN.K2-$ASPEN_V-ZHEMVL-$GPU$OPTIONAL
$EXEC ./test2-uhemv-u $IN | tee log-ASPEN.K2-$ASPEN_V-UHEMVU-$GPU$OPTIONAL
$EXEC ./test2-uhemv-l $IN | tee log-ASPEN.K2-$ASPEN_V-UHEMVL-$GPU$OPTIONAL
fi

if [ $EXEC_CUDA  -ne 0 ]; then
$EXEC ./cuda-ssymv-u $IN 1 | tee log-CUDA-$CUDA_V-SSYMVU-$GPU$OPTIONAL
$EXEC ./cuda-ssymv-l $IN 1 | tee log-CUDA-$CUDA_V-SSYMVL-$GPU$OPTIONAL
$EXEC ./cuda-ssymv-u $IN 0 | tee log-CUDA-$CUDA_V-SSYMVUnoAtomic-$GPU$OPTIONAL
$EXEC ./cuda-ssymv-l $IN 0 | tee log-CUDA-$CUDA_V-SSYMVLnoAtomic-$GPU$OPTIONAL
$EXEC ./cuda-dsymv-u $IN 1 | tee log-CUDA-$CUDA_V-DSYMVU-$GPU$OPTIONAL
$EXEC ./cuda-dsymv-l $IN 1 | tee log-CUDA-$CUDA_V-DSYMVL-$GPU$OPTIONAL
$EXEC ./cuda-dsymv-u $IN 0 | tee log-CUDA-$CUDA_V-DSYMVUnoAtomic-$GPU$OPTIONAL
$EXEC ./cuda-dsymv-l $IN 0 | tee log-CUDA-$CUDA_V-DSYMVLnoAtomic-$GPU$OPTIONAL
$EXEC ./cuda-chemv-u $IN 1 | tee log-CUDA-$CUDA_V-CHEMVU-$GPU$OPTIONAL
$EXEC ./cuda-chemv-l $IN 1 | tee log-CUDA-$CUDA_V-CHEMVL-$GPU$OPTIONAL
$EXEC ./cuda-chemv-u $IN 0 | tee log-CUDA-$CUDA_V-CHEMVUnoAtomic-$GPU$OPTIONAL
$EXEC ./cuda-chemv-l $IN 0 | tee log-CUDA-$CUDA_V-CHEMVLnoAtomic-$GPU$OPTIONAL
$EXEC ./cuda-zhemv-u $IN 1 | tee log-CUDA-$CUDA_V-ZHEMVU-$GPU$OPTIONAL
$EXEC ./cuda-zhemv-l $IN 1 | tee log-CUDA-$CUDA_V-ZHEMVL-$GPU$OPTIONAL
$EXEC ./cuda-zhemv-u $IN 0 | tee log-CUDA-$CUDA_V-ZHEMVUnoAtomic-$GPU$OPTIONAL
$EXEC ./cuda-zhemv-l $IN 0 | tee log-CUDA-$CUDA_V-ZHEMVLnoAtomic-$GPU$OPTIONAL
fi

if [ $EXEC_MAGMA -ne 0 ]; then
$EXEC ./magma-ssymv-u  $IN | tee log-MAGMA-$MAGMA_V-SSYMVU-$GPU$OPTIONAL
$EXEC ./magma-ssymv-l  $IN | tee log-MAGMA-$MAGMA_V-SSYMVL-$GPU$OPTIONAL
$EXEC ./magma2-ssymv-u $IN | tee log-MAGMA2-$MAGMA_V-SSYMVU-$GPU$OPTIONAL
$EXEC ./magma2-ssymv-l $IN | tee log-MAGMA2-$MAGMA_V-SSYMVL-$GPU$OPTIONAL
$EXEC ./magma-dsymv-u  $IN | tee log-MAGMA-$MAGMA_V-DSYMVU-$GPU$OPTIONAL
$EXEC ./magma-dsymv-l  $IN | tee log-MAGMA-$MAGMA_V-DSYMVL-$GPU$OPTIONAL
$EXEC ./magma2-dsymv-u $IN | tee log-MAGMA2-$MAGMA_V-DSYMVU-$GPU$OPTIONAL
$EXEC ./magma2-dsymv-l $IN | tee log-MAGMA2-$MAGMA_V-DSYMVL-$GPU$OPTIONAL
$EXEC ./magma-chemv-u  $IN | tee log-MAGMA-$MAGMA_V-CHEMVU-$GPU$OPTIONAL
$EXEC ./magma-chemv-l  $IN | tee log-MAGMA-$MAGMA_V-CHEMVL-$GPU$OPTIONAL
$EXEC ./magma2-chemv-u $IN | tee log-MAGMA2-$MAGMA_V-CHEMVU-$GPU$OPTIONAL
$EXEC ./magma2-chemv-l $IN | tee log-MAGMA2-$MAGMA_V-CHEMVL-$GPU$OPTIONAL
$EXEC ./magma-zhemv-u  $IN | tee log-MAGMA-$MAGMA_V-ZHEMVU-$GPU$OPTIONAL
$EXEC ./magma-zhemv-l  $IN | tee log-MAGMA-$MAGMA_V-ZHEMVL-$GPU$OPTIONAL
$EXEC ./magma2-zhemv-u $IN | tee log-MAGMA2-$MAGMA_V-ZHEMVU-$GPU$OPTIONAL
$EXEC ./magma2-zhemv-l $IN | tee log-MAGMA2-$MAGMA_V-ZHEMVL-$GPU$OPTIONAL
fi

if [ $EXEC_KBLAS -ne 0 ]; then
$EXEC ./kblas-ssymv-u $IN | tee log-KBLAS-$KBLAS_V-SSYMVU-$GPU$OPTIONAL
$EXEC ./kblas-ssymv-l $IN | tee log-KBLAS-$KBLAS_V-SSYMVL-$GPU$OPTIONAL
$EXEC ./kblas-dsymv-u $IN | tee log-KBLAS-$KBLAS_V-DSYMVU-$GPU$OPTIONAL
$EXEC ./kblas-dsymv-l $IN | tee log-KBLAS-$KBLAS_V-DSYMVL-$GPU$OPTIONAL
fi

exit 0

