#ifndef	ASPEN_H_INCLUDED

#include "aspen_version.h"
#include "aspen_types.h"


#ifdef __cplusplus
extern "C" {
#endif
  
  
  /* User must call ASPEN_init for initialization */
  int
  ASPEN_init ( const int device_id );
  
  /* User must finalize the ASPEN library by ASPEN_shutdown */
  int
  ASPEN_shutdown ( void );

  /* Get version info ASPEN_get_version_info */
  void 
  ASPEN_get_version_info ( int &version, char *codename, char *releasedate );
  
  
  void
  ASPEN_ssymv ( char uplo, int n,
                float alpha, 
                const float * a, int lda,
                const float * x, int incx,
                float beta,  
                float * y, int incy );
  void
  ASPEN_dsymv ( char uplo, int n,
                double alpha, 
                const double * a, int lda,
                const double * x, int incx,
                double beta,  
                double * y, int incy );

  void
  ASPEN_chemv ( char uplo, int n,
                cuFloatComplex alpha, 
                const cuFloatComplex * a, int lda,
                const cuFloatComplex * x, int incx,
                cuFloatComplex beta,  
                cuFloatComplex * y, int incy );
  void
  ASPEN_zhemv ( char uplo, int n,
                cuDoubleComplex alpha, 
                const cuDoubleComplex * a, int lda,
                const cuDoubleComplex * x, int incx,
                cuDoubleComplex beta,  
                cuDoubleComplex * y, int incy );

  void
  ASPEN_wsymv ( char uplo, int n,
                cuddreal alpha, 
                const cuddreal * a, int lda,
                const cuddreal * x, int incx,
                cuddreal beta,  
                cuddreal * y, int incy );

  void
  ASPEN_uhemv ( char uplo, int n,
                cuddcomplex alpha, 
                const cuddcomplex * a, int lda,
                const cuddcomplex * x, int incx,
                cuddcomplex beta,  
                cuddcomplex * y, int incy );



#if ASPEN_HALF_ENABLED
  void
  ASPEN_haxpy ( int n,
                half alpha,
                half * x, int incx,
                half * y, int incy );
#endif
  void
  ASPEN_saxpy ( int n,
                float alpha,
                float * x, int incx,
                float * y, int incy );
  void
  ASPEN_daxpy ( int n,
                double alpha,
                double * x, int incx,
                double * y, int incy );
  void
  ASPEN_caxpy ( int n,
                cuFloatComplex alpha,
                cuFloatComplex * x, int incx,
                cuFloatComplex * y, int incy );
  void
  ASPEN_zaxpy ( int n,
                cuDoubleComplex alpha,
                cuDoubleComplex * x, int incx,
                cuDoubleComplex * y, int incy );
  void
  ASPEN_waxpy ( int n,
                cuddreal alpha,
                cuddreal * x, int incx,
                cuddreal * y, int incy );
  void
  ASPEN_uaxpy ( int n,
                cuddcomplex alpha,
                cuddcomplex * x, int incx,
                cuddcomplex * y, int incy );
 

#if ASPEN_HALF_ENABLED
  void
  ASPEN_haxpby ( int n,
                 half alpha,
                 half * x, int incx,
                 half beta,
                 half * y, int incy );
#endif
  void
  ASPEN_saxpby ( int n,
                 float alpha,
                 float * x, int incx,
                 float beta,
                 float * y, int incy );
  void
  ASPEN_daxpby ( int n,
                 double alpha,
                 double * x, int incx,
                 double beta,
                 double * y, int incy );
  void
  ASPEN_caxpby ( int n,
                 cuFloatComplex alpha,
                 cuFloatComplex * x, int incx,
                 cuFloatComplex beta,
                 cuFloatComplex * y, int incy );
  void
  ASPEN_zaxpby ( int n,
                 cuDoubleComplex alpha,
                 cuDoubleComplex * x, int incx,
                 cuDoubleComplex beta,
                 cuDoubleComplex * y, int incy );
  void
  ASPEN_waxpby ( int n,
                 cuddreal alpha,
                 cuddreal * x, int incx,
                 cuddreal beta,
                 cuddreal * y, int incy );
  void
  ASPEN_uaxpby ( int n,
                 cuddcomplex alpha,
                 cuddcomplex * x, int incx,
                 cuddcomplex beta,
                 cuddcomplex * y, int incy );


#if ASPEN_HALF_ENABLED
  void
  ASPEN_hswap ( int n,
                half * x, int incx,
                half * y, int incy );
#endif
  void
  ASPEN_sswap ( int n,
                float * x, int incx,
                float * y, int incy );
  void
  ASPEN_dswap ( int n,
                double * x, int incx,
                double * y, int incy );
  void
  ASPEN_cswap ( int n,
                cuFloatComplex * x, int incx,
                cuFloatComplex * y, int incy );
  void
  ASPEN_zswap ( int n,
                cuDoubleComplex * x, int incx,
                cuDoubleComplex * y, int incy );
  void
  ASPEN_wswap ( int n,
                cuddreal * x, int incx,
                cuddreal * y, int incy );
  void
  ASPEN_uswap ( int n,
                cuddcomplex * x, int incx,
                cuddcomplex * y, int incy );


#if ASPEN_HALF_ENABLED
  void
  ASPEN_hcopy ( int n,
                half * x, int incx,
                half * y, int incy );
#endif
  void
  ASPEN_scopy ( int n,
                float * x, int incx,
                float * y, int incy );
  void
  ASPEN_dcopy ( int n,
                double * x, int incx,
                double * y, int incy );
  void
  ASPEN_ccopy ( int n,
                cuFloatComplex * x, int incx,
                cuFloatComplex * y, int incy );
  void
  ASPEN_zcopy ( int n,
                cuDoubleComplex * x, int incx,
                cuDoubleComplex * y, int incy );
  void
  ASPEN_wcopy ( int n,
                cuddreal * x, int incx,
                cuddreal * y, int incy );
  void
  ASPEN_ucopy ( int n,
                cuddcomplex * x, int incx,
                cuddcomplex * y, int incy );

#if ASPEN_HALF_ENABLED
  void
  ASPEN_hscal ( int n,
                half alpha,
                half * x, int incx );
#endif
  void
  ASPEN_sscal ( int n,
                float alpha,
                float * x, int incx );
  void
  ASPEN_dscal ( int n,
                double alpha,
                double * x, int incx );
  void
  ASPEN_cscal ( int n,
                cuFloatComplex alpha,
                cuFloatComplex * x, int incx );
  void
  ASPEN_zscal ( int n,
                cuDoubleComplex alpha,
                cuDoubleComplex * x, int incx );
  void
  ASPEN_wscal ( int n,
                cuddreal alpha,
                cuddreal * x, int incx );
  void
  ASPEN_uscal ( int n,
                cuddcomplex alpha,
                cuddcomplex * x, int incx );
  

#if ASPEN_HALF_ENABLED
  void
  ASPEN_HZERO ( half * x, int n );
#endif
  void
  ASPEN_SZERO ( float * x, int n );
  void
  ASPEN_DZERO ( double * x, int n );
  void
  ASPEN_CZERO ( cuFloatComplex * x, int n );
  void
  ASPEN_ZZERO ( cuDoubleComplex * x, int n );
  void
  ASPEN_WZERO ( cuddreal * x, int n );
  void
  ASPEN_UZERO ( cuddcomplex * x, int n );
 

#ifdef __cplusplus
}
#endif

#define	ASPEN_H_INCLUDED	1
#endif

