#include "aspen_real.h"
#ifndef ASPEN_DFREAL_H_INCLUDED

struct __align__(8) __cudfreal__
{
  float x, y;
};

typedef	struct __cudfreal__		cudfreal;


#ifdef __cplusplus

#ifdef __CUDA_ARCH__
#if GPU_ARCH>300
__device__ __forceinline__ cudfreal
__ldg ( const cudfreal * a )
{
  cudfreal t;
  t.x = __ldg( &(a->x) ); t.y = __ldg( &(a->y) );
  return t;
}
#endif
#endif


__host__ __device__ __forceinline__ bool
operator== ( const cudfreal a, const cudfreal b )
{
  if ((a.x == b.x) && (a.y == b.y))
    return true;
  else
    return false;
}

__host__ __device__ __forceinline__ bool
operator!= ( const cudfreal a, const cudfreal b)
{
  if ((a.x != b.x) || (a.y != b.y))
    return true;
  else
    return false;
}

__host__ __device__ __forceinline__ bool
operator< ( const cudfreal a, const cudfreal b)
{
  bool ret;
  if ( a.x == b.x ) {
    ret = (a.y < b.y) ? true : false;
  } else {
    ret = (a.x < b.x) ? true : false;
  }
  return ret;
}

__host__ __device__ __forceinline__ bool
operator> ( const cudfreal a, const cudfreal b)
{
  return ( b < a );
}

__host__ __device__ __forceinline__ bool
operator<= ( const cudfreal a, const cudfreal b)
{
  bool ret;
  if ( a.x == b.x ) {
    ret = (a.y <= b.y) ? true : false;
  } else {
    ret = (a.x < b.x) ? true : false;
  }
  return ret;
}

__host__ __device__ __forceinline__ bool
operator>= ( const cudfreal a, const cudfreal b)
{
  return ( b <= a );
}


__host__ __device__ __forceinline__ float
__fmad ( const float a,  const float b, const float c )
{
  return  fma ( a, b, c );
}


__host__ __device__ __forceinline__ void
cuWTwoSum ( const float a, const float b, float &s, float &e )
{
  float v;
  s = __add ( a, b );
  v = __sub ( s, a );
  e = __add ( __sub ( a, __sub ( s, v ) ), __sub ( b, v ) );
}

__host__ __device__ __forceinline__ void
cuWQuickTwoSum ( const float a, const float b, float &s, float &e )
{
  s = __add ( a, b );
  e = __sub ( b, __sub ( s, a ) );
}


__host__ __device__ __forceinline__ cudfreal
CUWADD ( const cudfreal a, const cudfreal b )
{
  cudfreal t, c;
  cuWTwoSum ( a.x, b.x, t.x, t.y );
  t.y = __add ( t.y, __add( a.y, b.y ) );
  cuWQuickTwoSum ( t.x, t.y, c.x, c.y );
  return c;
}

__host__ __device__ __forceinline__ cudfreal
CUWSUB ( const cudfreal a, const cudfreal b )
{
  cudfreal t, c;
  cuWTwoSum ( a.x, -b.x, t.x, t.y );
  t.y = __add ( t.y, __sub ( a.y, b.y ) );
  cuWQuickTwoSum ( t.x, t.y, c.x, c.y );
  return c;
}

__host__ __device__ __forceinline__ cudfreal
CUWMUL ( const cudfreal a, const cudfreal b )
{
  float t, e;
  cudfreal c;
  t   = __fmad ( a.y, b.x, a.x * b.y );
  c.x = __fmad ( a.x, b.x, t );
  e   = __fmad ( a.x, b.x, -c.x );
  c.y = __add ( e, t );
  return c;
}

__host__ __device__ __forceinline__ cudfreal
CUSMUL ( const cudfreal a, const float b )
{
  float t, e;
  cudfreal c;
  t   = __mul ( a.y, b );
  c.x = __fmad ( a.x, b, t );
  e   = __fmad ( a.x, b, -c.x );
  c.y = __add ( e, t );
  return c;
}


__host__ __device__ __forceinline__ cudfreal
operator+ ( const cudfreal a )
{
  return ( a );
}

__host__ __device__ __forceinline__ cudfreal
operator+ ( const cudfreal a, const cudfreal b )
{
  return CUWADD ( a, b );
}

__host__ __device__ __forceinline__ void
operator+= ( cudfreal &a, const cudfreal b )
{
  a = CUWADD ( a, b );
}

__host__ __device__ __forceinline__ cudfreal
operator- ( const cudfreal a )
{
  cudfreal  t; t.x = -a.x; t.y = -a.y;
  return ( t );
}

__host__ __device__ __forceinline__ cudfreal
operator- ( const cudfreal a, const cudfreal b )
{
  return CUWSUB ( a, b );
}

__host__ __device__ __forceinline__ void
operator-= ( cudfreal &a, const cudfreal b )
{
  a = CUWSUB ( a, b );
}

__host__ __device__ __forceinline__ cudfreal
operator* ( const cudfreal a, const cudfreal b )
{
  return CUWMUL ( a, b );
}

__host__ __device__ __forceinline__ cudfreal
operator* ( const cudfreal a, const float b )
{
  return CUSMUL ( a, b );
}

__host__ __device__ __forceinline__ cudfreal
operator* ( const float a, const cudfreal b )
{
  return CUSMUL ( b, a );
}

__host__ __device__ __forceinline__ void
operator*= ( cudfreal &a, const cudfreal b )
{
  a =  ( a * b );
}

__host__ __device__ __forceinline__ void
operator*= ( cudfreal &a, const float b )
{
  a = ( a * b );
}

__host__ __device__ __forceinline__ cudfreal
fma ( const cudfreal a, const cudfreal b, const cudfreal c )
{
  cudfreal  t = (a * b) + c;
  return t;
}

__host__ __device__ __forceinline__ cudfreal
CUWDIV ( const cudfreal a, const cudfreal b )
{
  const cudfreal ZERO = { (float)0, (float)0 };
  if ( b == ZERO ) {
    const cudfreal NAN_DD = { NAN, NAN };
    return NAN_DD;
  }
  if ( a == ZERO ) { return ZERO; }
  const cudfreal ONE  = { (float)1, (float)0 };
  if ( b == ONE  ) { return a; }
  const cudfreal MONE = { (float)-1, (float)0 };
  if ( b == MONE ) { return -a; }

  /* A.H.Karp and P.Markstein:
   * High Precision Division and Square Root,
   * ATOMS, 23(4), 1997, p.561-589.
   *
   *  when x=Approx(a/b),
   *  y = b*x
   *  Y = y + x(b-a*y) = goodApprox(a/b)
   */
  const float    xn = a.x / b.x;
  const cudfreal yn = b * xn;
  const cudfreal T  = a * yn;
  cudfreal       t  = b - T;
  t *= xn;
  const cudfreal Y  = yn + t;

  return Y;
}

__host__ __device__ __forceinline__ cudfreal
operator/ ( const cudfreal a, const cudfreal b )
{
  const cudfreal t = CUWDIV ( a, b );
  return t;
}

__host__ __device__ __forceinline__ void
operator/= ( cudfreal &a, const cudfreal b )
{
  a = ( a / b );
}


__host__ __device__ __forceinline__ cudfreal
Abs ( const cudfreal a )
{
  const float tx = fabsf(a.x);
  const float ty = fabsf(a.y);
  const cudfreal  t = { tx, ty };
  return t;
}

__host__ __device__ __forceinline__ cudfreal
Conj ( const cudfreal a )
{
  return a;
}


__host__ __device__ __forceinline__ cudfreal
Sqrt ( const cudfreal a )
{
  const cudfreal ZERO = { (float)0, (float)0 };
  if ( a == ZERO ) { return a; }
  if ( a < ZERO ) {
    const cudfreal NAN_DD = { NAN, NAN };
    return NAN_DD;
  }

  /* A.H.Karp and P.Markstein:
   * High Precision Division and Square Root,
   * ATOMS, 23(4), 1997, p.561-589.
   *
   *  when x=Approx(1/sqrt(a)),
   *  y = a*x
   *  Y = y + (x/2)(a-y^2) = goodApprox(sqrt(a))
   */
  float          xn = 1e0 / sqrtf(a.x);
  const cudfreal yn = a * xn;
  const cudfreal T  = yn * yn;
  cudfreal       t  = a - T;
  xn /= 2; t *= xn;
  const cudfreal Y  = yn + t;

  return Y;
}

__host__ __device__ __forceinline__ cudfreal
__choose__ ( const bool flag, const cudfreal a, const cudfreal b )
{
  const float  tx = (flag ? a.x : b.x);
  const float  ty = (flag ? a.y : b.y);
  const cudfreal  t = { tx, ty };
  return t;
}


#endif

#define ASPEN_DFREAL_H_INCLUDED		1
#endif

