#ifndef ASPEN_LDST_H_INCLUDED

#include "aspen_types.h"

// ---------------------------------------------------------
__forceinline__ __device__ cuddreal
Load_Global ( cuddreal *addr__ )
{
  double  sx,sy;
  asm volatile ( "ld.global.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  cuddreal  s = { sx, sy };
  return s;
}

__forceinline__ __device__ cudfreal
Load_Global ( cudfreal *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.global.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cudfreal t = { sx, sy };
  return t;
}

__forceinline__ __device__ double
Load_Global ( double *addr__ )
{
  double  s;
  asm volatile ( "ld.global.f64\t%0, [%1];"
                 : "=d"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ float
Load_Global ( float *addr__ )
{
  float  s;
  asm volatile ( "ld.global.f32\t%0, [%1];"
                 : "=f"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ cuddcomplex
Load_Global ( cuddcomplex *addr__ )
{
  const cuddreal sx = Load_Global ( &(addr__->x) );
  const cuddreal sy = Load_Global ( &(addr__->y) );
  const cuddcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cudfcomplex
Load_Global ( cudfcomplex *addr__ )
{
  const cudfreal sx = Load_Global ( &(addr__->x) );
  const cudfreal sy = Load_Global ( &(addr__->y) );
  const cudfcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuDoubleComplex
Load_Global ( cuDoubleComplex *addr__ )
{
  double  sx,sy;
  asm volatile ( "ld.global.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  const cuDoubleComplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuFloatComplex
Load_Global ( cuFloatComplex *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.global.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cuFloatComplex t = { sx, sy };
  return t;
}
// ---------------------------------------------------------

// ---------------------------------------------------------
__forceinline__ __device__ cuddreal
Load_Global_CG ( cuddreal *addr__ )
{
  double  sx, sy;
  asm volatile ( "ld.global.cg.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  cuddreal  s = { sx, sy };
  return s;
}

__forceinline__ __device__ cudfreal
Load_Global_CG ( cudfreal *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.global.cg.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cudfreal t = { sx, sy };
  return t;
}

__forceinline__ __device__ double
Load_Global_CG ( double *addr__ )
{
  double  s;
  asm volatile ( "ld.global.cg.f64\t%0, [%1];"
                 : "=d"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ float
Load_Global_CG ( float *addr__ )
{
  float  s;
  asm volatile ( "ld.global.cg.f32\t%0, [%1];"
                 : "=f"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ cuddcomplex
Load_Global_CG ( cuddcomplex *addr__ )
{
  const cuddreal sx = Load_Global_CG ( &(addr__->x) );
  const cuddreal sy = Load_Global_CG ( &(addr__->y) );
  const cuddcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cudfcomplex
Load_Global_CG ( cudfcomplex *addr__ )
{
  const cudfreal sx = Load_Global_CG ( &(addr__->x) );
  const cudfreal sy = Load_Global_CG ( &(addr__->y) );
  const cudfcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuDoubleComplex
Load_Global_CG ( cuDoubleComplex *addr__ )
{
  double  sx,sy;
  asm volatile ( "ld.global.cg.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  const cuDoubleComplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuFloatComplex
Load_Global_CG ( cuFloatComplex *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.global.cg.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cuFloatComplex t = { sx, sy };
  return t;
}
// ---------------------------------------------------------

// ---------------------------------------------------------
__forceinline__ __device__ cuddreal
Load_Global_CS ( cuddreal *addr__ )
{
  double  sx, sy;
  asm volatile ( "ld.global.cs.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  cuddreal  s = { sx, sy };
  return s;
}

__forceinline__ __device__ cudfreal
Load_Global_CS ( cudfreal *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.global.cs.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cudfreal t = { sx, sy };
  return t;
}

__forceinline__ __device__ double
Load_Global_CS ( double *addr__ )
{
  double  s;
  asm volatile ( "ld.global.cs.f64\t%0, [%1];"
                 : "=d"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ float
Load_Global_CS ( float *addr__ )
{
  float  s;
  asm volatile ( "ld.global.cs.f32\t%0, [%1];"
                 : "=f"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ cuddcomplex
Load_Global_CS ( cuddcomplex *addr__ )
{
  const cuddreal sx = Load_Global_CS ( &(addr__->x) );
  const cuddreal sy = Load_Global_CS ( &(addr__->y) );
  const cuddcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cudfcomplex
Load_Global_CS ( cudfcomplex *addr__ )
{
  const cudfreal sx = Load_Global_CS ( &(addr__->x) );
  const cudfreal sy = Load_Global_CS ( &(addr__->y) );
  const cudfcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuDoubleComplex
Load_Global_CS ( cuDoubleComplex *addr__ )
{
  double  sx, sy;
  asm volatile ( "ld.global.cs.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  const cuDoubleComplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuFloatComplex
Load_Global_CS ( cuFloatComplex *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.global.cs.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cuFloatComplex t = { sx, sy };
  return t;
}
// ---------------------------------------------------------

// ---------------------------------------------------------
__forceinline__ __device__ cuddreal
Load_Global_CV ( cuddreal *addr__ )
{
  double  sx, sy;
  asm volatile ( "ld.global.cv.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  cuddreal s = {sx, sy};
  return s;
}

__forceinline__ __device__ cudfreal
Load_Global_CV ( cudfreal *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.global.cv.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cudfreal t = { sx, sy };
  return t;
}

__forceinline__ __device__ double
Load_Global_CV ( double *addr__ )
{
  double  s;
  asm volatile ( "ld.global.cv.f64\t%0, [%1];"
                 : "=d"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ float
Load_Global_CV ( float *addr__ )
{
  float  s;
  asm volatile ( "ld.global.cv.f32\t%0, [%1];"
                 : "=f"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ cuddcomplex
Load_Global_CV ( cuddcomplex *addr__ )
{
  const cuddreal sx = Load_Global_CV ( &(addr__->x) );
  const cuddreal sy = Load_Global_CV ( &(addr__->y) );
  const cuddcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cudfcomplex
Load_Global_CV ( cudfcomplex *addr__ )
{
  const cudfreal sx = Load_Global_CV ( &(addr__->x) );
  const cudfreal sy = Load_Global_CV ( &(addr__->y) );
  const cudfcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuDoubleComplex
Load_Global_CV ( cuDoubleComplex *addr__ )
{
  double  sx,sy;
  asm volatile ( "ld.global.cv.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  const cuDoubleComplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuFloatComplex
Load_Global_CV ( cuFloatComplex *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.global.cv.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cuFloatComplex t = { sx, sy };
  return t;
}
// ---------------------------------------------------------

// ---------------------------------------------------------
__forceinline__ __device__ cuddreal
Load_Volatile_Global ( cuddreal *addr__ )
{
  double  sx, sy;
  asm volatile ( "ld.volatile.global.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  cuddreal  s = { sx, sy };
  return s;
}

__forceinline__ __device__ cudfreal
Load_Volatile_Global ( cudfreal *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.volatile.global.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cudfreal t = { sx, sy };
  return t;
}

__forceinline__ __device__ double
Load_Volatile_Global ( double *addr__ )
{
  double  s;
  asm volatile ( "ld.volatile.global.f64\t%0, [%1];"
                 : "=d"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ float
Load_Volatile_Global ( float *addr__ )
{
  float  s;
  asm volatile ( "ld.volatile.global.f32\t%0, [%1];"
                 : "=f"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ cuddcomplex
Load_Volatile_Global ( cuddcomplex *addr__ )
{
  const cuddreal sx = Load_Volatile_Global ( &(addr__->x) );
  const cuddreal sy = Load_Volatile_Global ( &(addr__->y) );
  const cuddcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cudfcomplex
Load_Volatile_Global ( cudfcomplex *addr__ )
{
  const cudfreal sx = Load_Volatile_Global ( &(addr__->x) );
  const cudfreal sy = Load_Volatile_Global ( &(addr__->y) );
  const cudfcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuDoubleComplex
Load_Volatile_Global ( cuDoubleComplex *addr__ )
{
  double  sx,sy;
  asm volatile ( "ld.volatile.global.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  const cuDoubleComplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuFloatComplex
Load_Volatile_Global ( cuFloatComplex *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.volatile.global.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cuFloatComplex t = { sx, sy };
  return t;
}
// ---------------------------------------------------------

// ---------------------------------------------------------
__forceinline__ __device__ cuddreal
Load_Volatile ( cuddreal *addr__ )
{
  double  sx, sy;
  asm volatile ( "ld.volatile.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  cuddreal  s = { sx, sy };
  return s;
}

__forceinline__ __device__ cudfreal
Load_Volatile ( cudfreal *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.volatile.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cudfreal t = { sx, sy };
  return t;
}

__forceinline__ __device__ double
Load_Volatile ( double *addr__ )
{
  double  s;
  asm volatile ( "ld.volatile.f64\t%0, [%1];"
                 : "=d"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ float
Load_Volatile ( float *addr__ )
{
  float  s;
  asm volatile ( "ld.volatile.f32\t%0, [%1];"
                 : "=f"(s) : "l"(addr__) );
  return s;
}

__forceinline__ __device__ cuddcomplex
Load_Volatile ( cuddcomplex *addr__ )
{
  const cuddreal sx = Load_Volatile ( &(addr__->x) );
  const cuddreal sy = Load_Volatile ( &(addr__->y) );
  const cuddcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cudfcomplex
Load_Volatile ( cudfcomplex *addr__ )
{
  const cudfreal sx = Load_Volatile ( &(addr__->x) );
  const cudfreal sy = Load_Volatile ( &(addr__->y) );
  const cudfcomplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuDoubleComplex
Load_Volatile ( cuDoubleComplex *addr__ )
{
  double  sx,sy;
  asm volatile ( "ld.volatile.v2.f64\t{%0,%1}, [%2];"
                 : "=d"(sx), "=d"(sy) : "l"(&addr__->x) );
  const cuDoubleComplex t = { sx, sy };
  return t;
}

__forceinline__ __device__ cuFloatComplex
Load_Volatile ( cuFloatComplex *addr__ )
{
  float  sx,sy;
  asm volatile ( "ld.volatile.v2.f32\t{%0,%1}, [%2];"
                 : "=f"(sx), "=f"(sy) : "l"(&addr__->x) );
  const cuFloatComplex t = { sx, sy };
  return t;
}
// ---------------------------------------------------------

// ---------------------------------------------------------
__forceinline__ __device__ void
Store_Global ( cuddreal *addr__, const cuddreal s )
{
  asm volatile ( "st.global.v2.f64\t[%2], {%0,%1};"
                 : : "d"(s.x), "d"(s.y), "l"(&addr__->x) );
}

__forceinline__ __device__ void
Store_Global ( cudfreal *addr__, const cudfreal s )
{
  asm volatile ( "st.global.v2.f32\t[%2], {%0,%1};"
                 : : "f"(s.x), "f"(s.y),"l"(&addr__->x) );
}

__forceinline__ __device__ void
Store_Global ( double *addr__, const double s )
{
  asm volatile ( "st.global.f64\t[%1], %0;"
                 : : "d"(s), "l"(addr__) );
}

__forceinline__ __device__ void
Store_Global ( float *addr__, const float s )
{
  asm volatile ( "st.global.f32\t[%1], %0;"
                 : : "f"(s), "l"(addr__) );
}

__forceinline__ __device__ void
Store_Global ( cuddcomplex *addr__, const cuddcomplex s )
{
  Store_Global ( &(addr__->x), s.x );
  Store_Global ( &(addr__->y), s.y );
}

__forceinline__ __device__ void
Store_Global ( cudfcomplex *addr__, const cudfcomplex s )
{
  Store_Global ( &(addr__->x), s.x );
  Store_Global ( &(addr__->y), s.y );
}

__forceinline__ __device__ void
Store_Global ( cuDoubleComplex *addr__, const cuDoubleComplex s )
{
  asm volatile ( "st.global.v2.f64\t[%2], {%0,%1};"
                 : : "d"(s.x), "d"(s.y), "l"(&addr__->x) );
}

// ---------------------------------------------------------

// ---------------------------------------------------------
__forceinline__ __device__ void
Store_Global_CG ( cuddreal *addr__, const cuddreal s )
{
  asm volatile ( "st.global.cg.v2.f64\t[%2], {%0,%1};"
                 : : "d"(s.x), "d"(s.y), "l"(&addr__->x) );
}

__forceinline__ __device__ void
Store_Global_CG ( cudfreal *addr__, const cudfreal s )
{
  asm volatile ( "st.global.cg.v2.f32\t[%2], {%0,%1};"
                 : : "f"(s.x), "f"(s.y), "l"(&addr__->x) );
}

__forceinline__ __device__ void
Store_Global_CG ( double *addr__, const double s )
{
  asm volatile ( "st.global.cg.f64\t[%1], %0;"
                 : : "d"(s), "l"(addr__) );
}

__forceinline__ __device__ void
Store_Global_CG ( float *addr__, const float s )
{
  asm volatile ( "st.global.cg.f32\t[%1], %0;"
                 : : "f"(s), "l"(addr__) );
}

__forceinline__ __device__ void
Store_Global_CG ( cuddcomplex *addr__, const cuddcomplex s )
{
  Store_Global_CG ( &(addr__->x), s.x );
  Store_Global_CG ( &(addr__->y), s.y );
}

__forceinline__ __device__ void
Store_Global_CG ( cudfcomplex *addr__, const cudfcomplex s )
{
  Store_Global_CG ( &(addr__->x), s.x );
  Store_Global_CG ( &(addr__->y), s.y );
}

__forceinline__ __device__ void
Store_Global_CG ( cuDoubleComplex *addr__, const cuDoubleComplex s )
{
  asm volatile ( "st.global.cg.v2.f64\t[%2], {%0,%1};"
                 : : "d"(s.x), "d"(s.y), "l"(&addr__->x) );
}

__forceinline__ __device__ void
Store_Global_CG ( cuFloatComplex *addr__, const cuFloatComplex s )
{
  asm volatile ( "st.global.cg.v2.f32\t[%2], {%0,%1};"
                 : : "f"(s.x), "f"(s.y), "l"(&addr__->x) );
}
// ---------------------------------------------------------

// ---------------------------------------------------------
__forceinline__ __device__ void
Store_Volatile_Global ( cuddreal *addr__, const cuddreal s )
{
  asm volatile ( "st.volatile.global.v2.f64\t[%2], {%0,%1};"
                 : : "d"(s.x), "d"(s.y), "l"(&addr__->x) );
}

__forceinline__ __device__ void
Store_Volatile_Global ( cudfreal *addr__, const cudfreal s )
{
  asm volatile ( "st.volatile.global.v2.f32\t[%2], {%0,%1};"
                 : : "f"(s.x), "f"(s.y), "l"(&addr__->x) );
}

__forceinline__ __device__ void
Store_Volatile_Global ( double *addr__, const double s )
{
  asm volatile ( "st.volatile.global.f64\t[%1], %0;"
                 : : "d"(s), "l"(addr__) );
}

__forceinline__ __device__ void
Store_Volatile_Global ( float *addr__, const float s )
{
  asm volatile ( "st.volatile.global.f32\t[%1], %0;"
                 : : "f"(s), "l"(addr__) );
}

__forceinline__ __device__ void
Store_Volatile_Global ( cuddcomplex *addr__, const cuddcomplex s )
{
  Store_Volatile_Global ( &(addr__->x), s.x );
  Store_Volatile_Global ( &(addr__->y), s.y );
}

__forceinline__ __device__ void
Store_Volatile_Global ( cudfcomplex *addr__, const cudfcomplex s )
{
  Store_Volatile_Global ( &(addr__->x), s.x );
  Store_Volatile_Global ( &(addr__->y), s.y );
}

__forceinline__ __device__ void
Store_Volatile_Global ( cuDoubleComplex *addr__, const cuDoubleComplex s )
{
  asm volatile ( "st.volatile.global.v2.f64\t[%2], {%0,%1};"
                 : : "d"(s.x), "d"(s.y), "l"(&addr__->x) );
}

__forceinline__ __device__ void
Store_Volatile_Global ( cuFloatComplex *addr__, const cuFloatComplex s )
{
  asm volatile ( "st.volatile.global.v2.f32\t[%2], {%0,%1};"
                 : : "f"(s.x), "f"(s.y), "l"(&addr__->x) );
}
// ---------------------------------------------------------

// ---------------------------------------------------------
__forceinline__ __device__ void
Store_Volatile ( cuddreal *addr__, const cuddreal s )
{
  asm volatile ( "st.volatile.v2.f64\t[%2], {%0,%1};"
                 : : "d"(s.x), "d"(s.y), "l"(&addr__->x) );
}

__forceinline__ __device__ void
Store_Volatile ( cudfreal *addr__, const cudfreal s )
{
  asm volatile ( "st.volatile.v2.f32\t[%2], {%0,%1};"
                 : : "f"(s.x), "f"(s.y), "l"(&addr__->x) );
}

__forceinline__ __device__ void
Store_Volatile ( double *addr__, const double s )
{
  asm volatile ( "st.volatile.f64\t[%1], %0;"
                 : : "d"(s), "l"(addr__) );
}

__forceinline__ __device__ void
Store_Volatile ( float *addr__, const float s )
{
  asm volatile ( "st.volatile.f32\t[%1], %0;"
                 : : "f"(s), "l"(addr__) );
}

__forceinline__ __device__ void
Store_Volatile ( cuddcomplex *addr__, const cuddcomplex s )
{
  Store_Volatile ( &(addr__->x), s.x );
  Store_Volatile ( &(addr__->y), s.y );
}

__forceinline__ __device__ void
Store_Volatile ( cudfcomplex *addr__, const cudfcomplex s )
{
  Store_Volatile ( &(addr__->x), s.x );
  Store_Volatile ( &(addr__->y), s.y );
}

__forceinline__ __device__ void
Store_Volatile ( cuDoubleComplex *addr__, const cuDoubleComplex s )
{
  asm volatile ( "st.volatile.v2.f64\t[%2], {%0,%1};"
                 : : "d"(s.x), "d"(s.y), "l"(&addr__->x) );
}

__forceinline__ __device__ void
Store_Volatile ( cuFloatComplex *addr__, const cuFloatComplex s )
{
  asm volatile ( "st.volatile.v2.f32\t[%2], {%0,%1};"
                 : : "f"(s.x), "f"(s.y), "l"(&addr__->x) );
}
// ---------------------------------------------------------

// ---------------------------------------------------------
__forceinline__ __device__ void *
convert_addr_to_global ( const void * addr__ )
{
  unsigned long long int  global_addr;
  asm volatile ( "cvta.to.global.u64\t%0, %1;"
                 : "=l"(global_addr) : "l"(addr__) );
  return (void *)global_addr;
}

__forceinline__ __device__ void
prefetch_L1 (  void *addr__ )
{
  const unsigned long  addr = (unsigned long) addr__;
  asm volatile ( "prefetch.global.L1\t[%0];"
                 : : "l"(addr) );
}

__forceinline__ __device__ void
prefetch_L2 (  void *addr__ )
{
  const unsigned long  addr = (unsigned long) addr__;
  asm volatile ( "prefetch.global.L2\t[%0];"
                 : : "l"(addr) );
}
// ---------------------------------------------------------

// ---------------------------------------------------------
__forceinline__ __device__ void
Store_Global_with_Confirm( cuddreal * addr, cuddreal x )
{
  while ( 1 ) {
    Store_Volatile_Global( addr, x );
    if ( x == Load_Global_CV ( addr ) ) break;
  }
}

__forceinline__ __device__ void
Store_Global_with_Confirm( cudfreal * addr, cudfreal x )
{
  while ( 1 ) {
    Store_Volatile_Global( addr, x );
    if ( x == Load_Global_CV ( addr ) ) break;
  }
}

__forceinline__ __device__ void 
Store_Global_with_Confirm( double * addr, double x )
{
  while ( 1 ) {
    Store_Volatile_Global( addr, x );
    if ( x == Load_Global_CV ( addr ) ) break;
  }
}

__forceinline__ __device__ void
Store_Global_with_Confirm( float * addr, float x )
{
  while ( 1 ) {
    Store_Volatile_Global( addr, x );
    if ( x == Load_Global_CV ( addr ) ) break;
  }
}

__forceinline__ __device__ void
Store_Global_with_Confirm( cuddcomplex * addr, cuddcomplex x )
{
  Store_Global_with_Confirm( &(addr->x), x.x );
  Store_Global_with_Confirm( &(addr->y), x.y );
}

__forceinline__ __device__ void
Store_Global_with_Confirm( cudfcomplex * addr, cudfcomplex x )
{
  Store_Global_with_Confirm( &(addr->x), x.x );
  Store_Global_with_Confirm( &(addr->y), x.y );
}

__forceinline__ __device__ void
Store_Global_with_Confirm( cuDoubleComplex * addr, cuDoubleComplex x )
{
  while ( 1 ) {
    Store_Volatile_Global( addr, x );
    if ( x == Load_Global_CV ( addr ) ) break;
  }
}

__forceinline__ __device__ void
Store_Global_with_Confirm( cuFloatComplex * addr, cuFloatComplex x )
{
  while ( 1 ) {
    Store_Volatile_Global( addr, x );
    if ( x == Load_Global_CV ( addr ) ) break;
  }
}
// ---------------------------------------------------------


#define ASPEN_LDST_H_INCLUDED		1
#endif

