#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdint.h>
#include "cublas.h"

#include "aspen.h"
#include "fortran.h"
#include "aspen_fortran.h"

// external API from fortran calls
extern "C" {

  void
  aspen_init_( const int *n )
  {
    ASPEN_init( *n );
  }

  void
  aspen_shutdown_()
  {
    ASPEN_shutdown( );
  }

  void
  aspen_get_version_info_ ( int *version,
                            char *codename, unsigned int length_codename,
                            char *releasedate, unsigned int length_releasedate )
  {
    int  version_;
    char code_name[256];
    char release_date[256];

    ASPEN_get_version_info ( version_, code_name, release_date );

    memset( codename, ' ', length_codename );
    size_t len_code_name = strlen( code_name );
    if ( len_code_name >= length_codename ) {
      len_code_name = (size_t)length_codename;
    }
    strncpy( codename, code_name, len_code_name );

    memset( releasedate, ' ', length_releasedate );
    size_t len_release_date = strlen( release_date );
    if ( len_release_date >= length_releasedate ) {
      len_release_date = (size_t)length_releasedate;
    }
    strncpy( releasedate, release_date, len_release_date );
  }

  void
  aspen_ssymv_(const char *uplo, const int *n,
               const float *alpha,
               const devptr_t *devPtrA, const int *lda,
               const devptr_t *devPtrx, const int *incx,
               const float *beta,
               const devptr_t *devPtry, const int *incy)
  {
    float *A = (float *)(uintptr_t)(*devPtrA);
    float *x = (float *)(uintptr_t)(*devPtrx);
    float *y = (float *)(uintptr_t)(*devPtry);

    ASPEN_ssymv( *uplo, *n,
                 *alpha, A, *lda, x, *incx, *beta,  y, *incy );
  }

  void
  aspen_dsymv_(const char *uplo, const int *n,
               const double *alpha,
               const devptr_t *devPtrA, const int *lda,
               const devptr_t *devPtrx, const int *incx,
               const double *beta,
               const devptr_t *devPtry, const int *incy)
  {
    double *A = (double *)(uintptr_t)(*devPtrA);
    double *x = (double *)(uintptr_t)(*devPtrx);
    double *y = (double *)(uintptr_t)(*devPtry);

    ASPEN_dsymv( *uplo, *n,
                 *alpha, A, *lda, x, *incx, *beta,  y, *incy );
  }

  void
  aspen_wsymv_(const char *uplo, const int *n,
               const cuddreal *alpha,
               const devptr_t *devPtrA, const int *lda,
               const devptr_t *devPtrx, const int *incx,
               const cuddreal *beta,
               const devptr_t *devPtry, const int *incy)
  {
    cuddreal *A = (cuddreal *)(uintptr_t)(*devPtrA);
    cuddreal *x = (cuddreal *)(uintptr_t)(*devPtrx);
    cuddreal *y = (cuddreal *)(uintptr_t)(*devPtry);

    ASPEN_wsymv( *uplo, *n,
                 *alpha, A, *lda, x, *incx, *beta,  y, *incy );
  }

  void
  aspen_chemv_(const char *uplo, const int *n,
               const cuFloatComplex *alpha,
               const devptr_t *devPtrA, const int *lda,
               const devptr_t *devPtrx, const int *incx,
               const cuFloatComplex *beta,
               const devptr_t *devPtry, const int *incy)
  {
    cuFloatComplex *A = (cuFloatComplex *)(uintptr_t)(*devPtrA);
    cuFloatComplex *x = (cuFloatComplex *)(uintptr_t)(*devPtrx);
    cuFloatComplex *y = (cuFloatComplex *)(uintptr_t)(*devPtry);

    ASPEN_chemv( *uplo, *n,
                 *alpha, A, *lda, x, *incx, *beta,  y, *incy );
  }

  void
  aspen_zhemv_(const char *uplo, const int *n,
               const cuDoubleComplex *alpha,
               const devptr_t *devPtrA, const int *lda,
               const devptr_t *devPtrx, const int *incx,
               const cuDoubleComplex *beta,
               const devptr_t *devPtry, const int *incy)
  {
    cuDoubleComplex *A = (cuDoubleComplex *)(uintptr_t)(*devPtrA);
    cuDoubleComplex *x = (cuDoubleComplex *)(uintptr_t)(*devPtrx);
    cuDoubleComplex *y = (cuDoubleComplex *)(uintptr_t)(*devPtry);

    ASPEN_zhemv( *uplo, *n,
                 *alpha, A, *lda, x, *incx, *beta,  y, *incy );
  }

  void
  aspen_uhemv_(const char *uplo, const int *n,
               const cuddcomplex *alpha,
               const devptr_t *devPtrA, const int *lda,
               const devptr_t *devPtrx, const int *incx,
               const cuddcomplex *beta,
               const devptr_t *devPtry, const int *incy)
  {
    cuddcomplex *A = (cuddcomplex *)(uintptr_t)(*devPtrA);
    cuddcomplex *x = (cuddcomplex *)(uintptr_t)(*devPtrx);
    cuddcomplex *y = (cuddcomplex *)(uintptr_t)(*devPtry);

    ASPEN_uhemv( *uplo, *n,
                 *alpha, A, *lda, x, *incx, *beta,  y, *incy );
  }



  void
  aspen_szero_(const devptr_t *devPtrA, const int *n)
  {
    float *A = (float *)(uintptr_t)(*devPtrA);
    ASPEN_SZERO( A, *n );
  }

  void
  aspen_dzero_(const devptr_t *devPtrA, const int *n)
  {
    double *A = (double *)(uintptr_t)(*devPtrA);
    ASPEN_DZERO( A, *n );
  }

  void
  aspen_wzero_(const devptr_t *devPtrA, const int *n)
  {
    cuddreal *A = (cuddreal *)(uintptr_t)(*devPtrA);
    ASPEN_WZERO( A, *n );
  }

  void
  aspen_czero_(const devptr_t *devPtrA, const int *n)
  {
    cuFloatComplex *A = (cuFloatComplex *)(uintptr_t)(*devPtrA);
    ASPEN_CZERO( A, *n );
  }

  void
  aspen_zzero_(const devptr_t *devPtrA, const int *n)
  {
    cuDoubleComplex *A = (cuDoubleComplex *)(uintptr_t)(*devPtrA);
    ASPEN_ZZERO( A, *n );
  }

  void
  aspen_uzero_(const devptr_t *devPtrA, const int *n)
  {
    cuddcomplex *A = (cuddcomplex *)(uintptr_t)(*devPtrA);
    ASPEN_UZERO( A, *n );
  }


}

