#!/bin/sh

echo_Message()
{
	echo '<--/****************************************'
	echo 'Automatic Performance Tuning for CHEMV-U'
	date
	echo -n 'Host on '; echo `hostname`
	echo 'Device is '$DEVICE
	if [ x${1} = xbegin ]; then
		echo 'Start.'
	fi
	if [ x${1} = xend ]; then
		echo 'Successfully completed.'
	fi
	if [ x${1} = xterminate ]; then
		echo 'Terminated unfortunately.'
	fi
	echo '****************************************/-->'
}


export LANG=C
LOG=log-chemvu-AT1X.`date +%Y.%m.%d.%T`
WORK_DIR=chemvu-current

if [ x${CUDA_PATH} != x ]; then
        export LD_LIBRARY_PATH=$CUDA_PATH/lib64:$LD_LIBRARY_PATH
fi
export LD_LIBRARY_PATH=`pwd`/../lib:$LD_LIBRARY_PATH

cd ../src; make get_dev_info; cd ../tuning
../src/get_dev_info $ASPEN_GPU_ID > DEV_INFO

DEVICE=`awk '/DEVICE=/{ print $2 }' DEV_INFO`
MP=`awk '/MP=/{ print $2 }' DEV_INFO`
CG=`awk '/CG=/{ print $2 }' DEV_INFO`
MAXDIM=`awk '/MAXDIM=/{ print $2 }' DEV_INFO`


cd ../src; make get_mult get_mult32 get_mult48; cd ../tuning

echo_Message begin

/bin/sh ./touch_header.sh

mkdir $LOG
if [ -f $WORK_DIR ]; then
	\rm -rf $WORK_DIR
fi
if [ -h $WORK_DIR ]; then
	\rm $WORK_DIR
fi
if [ -d $WORK_DIR ]; then
	\rm -rf $WORK_DIR
fi
ln -s $LOG $WORK_DIR

echo 'Tuning processes are recorded on '$LOG'.'

cd $WORK_DIR
cd ../../src; make get_dev_info; cd ../tuning/$WORK_DIR
ln -s ../../src/get_dev_info .

tar -C ../.. -cf - src template | tar -xf -

#cat symv_upper_atomic_template.incl

/bin/sh ../d.sh > OK-pat-symv

touch done_pattern
\rm done_pattern

/bin/sh ../chemvu_a.sh | tee $LOG
/bin/sh ../chemvu_b0.sh $LOG
/bin/sh ../chemvu_b1.sh $LOG-X
/bin/sh ../chemvu_b2.sh $LOG-X-X
/bin/sh ../chemvu_b3.sh $LOG-X-X-X
/bin/sh ../chemvu_c.sh
/bin/sh ../d_filter.sh chemv-upper-auto3.h chemv-upper-auto2.h

echo "Complete phase d"

echo '#if 0'             > chemv-upper-auto_.h
echo_Message            >> chemv-upper-auto_.h
cat ../DEV_INFO         >> chemv-upper-auto_.h
echo '<--'              >> chemv-upper-auto_.h
cat ../CURRENT_GPU      >> chemv-upper-auto_.h
echo '-->'              >> chemv-upper-auto_.h
echo '#endif'           >> chemv-upper-auto_.h
cat chemv-upper-auto.h  >> chemv-upper-auto_.h
mv chemv-upper-auto_.h chemv-upper-auto.h
cp chemv-upper-auto.h ..

echo '#if 0'             > chemv-upper-auto_.h
echo_Message            >> chemv-upper-auto_.h
cat ../DEV_INFO         >> chemv-upper-auto_.h
echo '<--'              >> chemv-upper-auto_.h
cat ../CURRENT_GPU      >> chemv-upper-auto_.h
echo '-->'              >> chemv-upper-auto_.h
echo '#endif'           >> chemv-upper-auto_.h
cat chemv-upper-auto2.h >> chemv-upper-auto_.h
mv chemv-upper-auto_.h chemv-upper-auto2.h
cp chemv-upper-auto2.h ..

cat chemv-upper-auto.h
cat chemv-upper-auto2.h

cd ../../src

\rm chemv_upper.cu_o
make

cd ../bench

\rm test-c.o test2-c.o
make 

echo "Complete phase e"

timeout -s KILL 4   ./test-chemv-u IN-medium >& /dev/null
timeout -s KILL 600 ./test-chemv-u IN-medium
if [ $? -eq 137 ]; then
timeout -s KILL 600 ./test-chemv-u IN-medium
fi

echo "Complete phase f1"

timeout -s KILL 4    ./test2-chemv-u IN-medium >& /dev/null
timeout -s KILL 3600 ./test2-chemv-u IN-medium
if [ $? -eq 137 ]; then
timeout -s KILL 3600 ./test2-chemv-u IN-medium
fi

echo "Complete phase f2"

cd ../tuning

touch .done-chemvu

echo_Message end

exit 0

