#!/bin/bash

export LD_LIBRARY_PATH=../../lib:$LD_LIBRARY_PATH

IN=$1

Make ()
{
	cd ../../src
	\rm chemv_upper.cu_o
	make
	cd ../bench
	\rm test-c.o test2-c.o
	make 
	cd ../tuning/chemvu-current
}

Do ()
{
	echo $IN
	awk -f ../anal_symv.awk $IN | \
	tee anal_symv-result | \
	awk ' \
		/N= '$N'/{ \
			print; \
		} \
	' | \
	sort -r | \
	head -200 | \
	awk ' \
		BEGIN { \
			N=0; Tmin=1.0; \
		} \
		{ \
			if ( N != $2 ) { \
				N=$2; Tmin=$3; p=1; \
				print $0" "300000; \
			} else { \
			T = (2.0e+3*(1.0*Tmin-$3)*(1.0*Tmin-$3))/(1.0*Tmin*Tmin); \
			T = T * sqrt(1.0*N)/100; \
                        if ( T < 700 ) { \
                           print $0" "(int(100000*exp(-T)+1)); \
                        } else { \
                           print $0" "1; \
                        } } \
			p++; \
		} \
	'
}

Main ()
{
N_PATTERN=`cat IN-exe-a | awk '{ if( i>0 && $1>0 ) printf("%d ",$1); i++; }'`
for N in \
	$N_PATTERN
do
	Do
done
}

Main | \
	awk '\
		{ \
			s=$4" "$5" "$6" "$7" "$8; \
			score[s] += $10; \
		} \
		END{ \
			for(f in score) { \
			if ( f ~ /^[ ]*$/ ) { \
				ff="32 1 1 1 1"; \
				printf("%08d %s\n", 0, ff); \
			} else {
				ff=f; \
				printf("%08d %s\n", score[f], f); \
			} \
			} \
			for(i=0;i<30;i++) { \
				printf("%08d %s\n", 0, ff); \
			} \
		}' | \
	sort -r | head -20 | tee top20-1step | \
	awk '\
		BEGIN{ \
			id=1; \
		}\
		{ \
			if ( $1 > 0 ) { \
				score[id]=$1; \
				BLOCK_SIZE[id]=$2; \
				VX[id]=$3; \
				UX[id]=$4; \
				MULTI[id]=$5; \
				MX[id]=$6; \
				id++; \
			} \
		} \
		END { \
			print "{"; \
			print "//=========="; \
			print "if(blk==-1){"; \
			print "\t#include \"chemv-upper-auto2.h\""; \
			print "}"; \
			print "//=========="; \
			print "#if 0"; \
			print "struct chemv_auto_tuning_param_t { "; \
			print "\tint\tBLOCK_SIZE, VX, UX, MULTI, MX; "; \
			print "} PARAM["id+1"] = {"; \
				for(i=1;i<id;i++) { \
					print "\t{ "BLOCK_SIZE[i]", "VX[i]", "UX[i]", "MULTI[i]", "MX[i]" },"; \
				} \
			print "\t{ 0, 0, 0, 0, 0 }\n};"; \
			print "#endif"; \
			print "//=========="; \
			print "switch ( BLK ) {"; \
			for(i=1;i<id;i++) { \
			for(m=0;m<40;m++) { \
				print "#if KERNEL_"i*40+m; \
				print "case "i*40+m":"; \
				print "\tAPI_private(HESYMVu_ATOMIC__host) < GPU_ARCH, scalar_t, "BLOCK_SIZE[i]", GY2D, "VX[i]", "UX[i]", "MULTI[i]", "m" >"; \
				print "\t\t( n, a, lda, x, y, alpha, beta ); break;"; \
				print "#endif"; \
			} \
			} \
			print "default:"; \
			print "\tbreak;"; \
			print "}"; \
			print "//=========="; \
			print "}"; \
		} \
	' > chemv-upper-auto.h
cp chemv-upper-auto.h ../


cat -n top20-1step

touch $IN-X
\rm $IN-X

	ID_max=16
if [ x$ASPEN_TUNING_LEVEL = xROUGH ]; then
	ID_max=10
fi
if [ x$ASPEN_TUNING_LEVEL = xFULL ]; then
	ID_max=20
fi
	ID_list=" 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 "

DONE_KERNEL=0
for ID in \
	$ID_list
do
  if [ $ID -le $ID_max ]; then

  POINT=`awk 'BEGIN{ X=0; }{ if (NR=='$ID' && $1~/[0-9]/) { X=$1;exit; } }END{ print X; }' top20-1step`
  if [ $POINT -gt 100 ]; then

\rm chemv-upper-auto2.h
touch chemv-upper-auto2.h
awk ' \
	BEGIN{ \
		for(i=0;i<40;i++) print "#define\tKERNEL_"'$ID'*40+i"\t1"; \
		exit; \
	} \
	' > chemv-upper-auto2.h
cp chemv-upper-auto2.h ../

Make >& /dev/null

#for M in \
#	0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29
for M in \
	0 1 10 11 20 21 30 31
do
	BLOCK_SIZE=`awk '{ i++; if (i=='$ID') { print $2; } }' top20-1step`
	VX=`awk '{ i++; if (i=='$ID') { print $3; } }' top20-1step`
	UX=`awk '{ i++; if (i=='$ID') { print $4; } }' top20-1step`
	MULTIPLICITY=`awk '{ i++; if (i=='$ID') { print $5; } }' top20-1step`

	if [ x$VX != x ]; then
		MM=`awk '{ if ( $1=='$UX'/'$VX' && $2==('$M'%10) ) { print "OK"; } }' OK-pat-symv`
        else
		MM=''
	fi

	if [ x$MM != x ]; then

	WW=`cat log-regs-$BLOCK_SIZE-$VX | \
                awk '/^[0-9]/{ if ( $3=='$UX' && $4=='$M' ) { if ( $NF >= '$MULTIPLICITY' && $8 > 0 ) print "OK"; exit; } }'`

        if [ x$WW != x ]; then

cat << EOF | tee -a $IN-X
#define	BLOCK_SIZE	$BLOCK_SIZE
#define	VX		$VX
#define	UX		$UX
#define	MULTIPLICITY	$MULTIPLICITY
#define	_M_		$M
EOF

	KERNEL=`expr $ID "*" 40`
	KERNEL=`expr $KERNEL "+" $M`
	timeout -s KILL 720 ../../bench/test2-chemv-u IN-exe-a $KERNEL | tee -a $IN-X
	echo $BLOCK_SIZE" "$VX" "$UX" "$MULTIPLICITY" "$M >> done_pattern
	DONE_KERNEL=`expr $DONE_KERNEL "+" 1`

	fi

	fi
done
  fi

  fi
done

echo "Complete phase b0"
